import os
from disqover import DisqoverSession, InlineCredentialsProvider
from .transfer_template_setting import AllTemplatesDownloader, AllUserViewTemplatesDownloader, \
    AllUserViewSettingsDownloader

os.environ['CURL_CA_BUNDLE'] = ""


plugin_interface_version = 2

def meta_data():
    return {
        'id': 'template_transfer',
        'name': 'Template Transfer (2.0)',
        'description': 'Execute a transfer of templates and/or settings from this location to a specified location',
        'type-id': 'event-processor',
    }


def session_options_definition():
    return [
        {
            'content_type': 'text',
            'id': 'destination',
            'name': 'URL Server Destination',
            'description': 'The destination URL to which the templates need to be copied.',
            'section': 'Destination Server'
        },
        {
            'content_type': 'text',
            'id': 'user',
            'name': 'User Name',
            'description': 'User that can log into the destination server.',
            'section': 'Destination Server'
        },
        {
            'content_type': 'text',
            'id': 'pwd',
            'name': 'User Password',
            'description': 'User pwd for the destination server.',
            'section': 'Destination Server'
        },
        {
            'content_type': 'text',
            'id': 'pwd',
            'name': 'User Password',
            'description': 'User pwd for the destination server.',
            'section': 'Transfer Options'
        },
        {
            'content_type': 'bool',
            'id': 'bool_copy_templates',
            'name': 'Copy Templates',
            'description': 'Copy Templates. If a User View is specified, this will copy the templates available for '
                           'that User View only.',
            'default': False,
            'section': 'Transfer Options'
        },
        {
            'content_type': 'single_choice',
            'id': 'template_mode',
            'name': 'Template Type',
            'description': 'Specify the type of the templates to be transferred. Choose \"Global\" for only '
                           'global templates, \"Local\" for only local templates, and \"All\" for both'
                           'global and local templates ',
            'choices': [{'id': 'all', 'name': 'All'},
                        {'id': 'global', 'name': 'Global'},
                        {'id': 'local', 'name': 'Local'}],
            'section': 'Transfer Options'
        },
        {
            'content_type': 'bool',
            'id': 'bool_template_setting',
            'name': 'Copy Template Settings',
            'description': 'Copy the templates system default settings. If a User View is specified, this will copy the'
                           'settings of that User View only.',
            'default': False,
            'section': 'Transfer Options'
        },
        {
            'content_type': 'bool',
            'id': 'bool_userview',
            'name': 'Copy for UserView',
            'description': 'Toggle on if you want to execute options above for specified User View.',
            'default': False,
            'section': 'Transfer Options'
        },
        {
            'content_type': 'text',
            'id': 'userview',
            'name': 'Select User View (optional)',
            'description': 'The name of the User View you want to copy the templates and/or settings for. '
                           'Leave black to copy all templates stored.',
            'default': '',
            'section': 'Transfer Options'
        },

    ]


def process_events(options_values, log, authenticator, **kwargs):
    # ignoring event details
    destination_url = options_values["destination"]
    user = options_values["user"]
    pwd = options_values["pwd"]

    copy_templates = options_values["bool_copy_templates"]
    copy_settings = options_values["bool_template_setting"]

    usrvw = options_values["userview"].lower()
    usrvw_specified = options_values["bool_userview"]


    session = DisqoverSession(authenticator)
    session.start()

    destination_creds = InlineCredentialsProvider(destination_url, user, pwd)
    new_session = DisqoverSession(destination_creds)
    new_session.start()

    if copy_templates:
        mode = options_values["template_mode"]
        log("Started streaming of the templates")
        if not usrvw_specified:
            streamer = AllTemplatesDownloader(session, new_session, mode, log)
        else:
            log(f"Selection for {usrvw} only")
            streamer = AllUserViewTemplatesDownloader(session, usrvw, new_session, mode, log)
        streamer.stream_individual_templates()
        log("Done streaming all of the templates")

    if copy_settings:
        log(f"Started streaming of template settings for {usrvw}")
        if usrvw_specified:
            streamer = AllUserViewSettingsDownloader(session, usrvw, new_session, log)
        streamer.stream_user_view_settings()
        log("Done streaming the template settings")


