import os
from disqover import DisqoverSession, InlineCredentialsProvider, DataQueryAPI
import requests

os.environ['CURL_CA_BUNDLE'] = ""

plugin_interface_version = 2

RESOURCE_TYPES = ["Search Page", "Dashboard", "Lens", "List"]
TYPE_TO_FILTER = {"Dashboard": {"type": "RESOURCE_TYPES.DASHBOARD_TEMPLATE"},
                  "Lens": {"type": "RESOURCE_TYPES.LENS_TEMPLATE", "lensLocation": "instance"},
                  "List": {"type": "RESOURCE_TYPES.LENS_TEMPLATE", "lensLocation": "instance_list"},
                  "Search Page": {"type": "RESOURCE_TYPES.SEARCH_PAGE_TEMPLATE"}}


class AllTemplatesDownloader():
    def __init__(self, session, new_session, mode, log):
        self.session = session
        self.new_session = new_session
        self.log = log
        self.mode = mode

    def stream_individual_templates(self):
        for resource_type in RESOURCE_TYPES:
            ids_names = self.get_ids(resource_type)
            for id, name in ids_names:
                self.get_content_by_id(id)
                self.stream_to_server(id, name)

    def get_content_by_id(self, id):
        self.content = self.session.get("/api/resources/" + id, None)["resources"]

    def get_cts(self):
        api = DataQueryAPI(self.session)
        self.log("Getting all the CTS")
        return [ct.uri for ct in api.configuration.canonical_types]

    def get_ids(self, resource_type):
        template_resource_type = TYPE_TO_FILTER[resource_type]
        # Use "all" for all templates, "global" for only global, "local" for personal
        query = {"offset": 0, "limit": 125, "filter": template_resource_type, "sort_by": "creationTimestamp",
                 "direction": "desc", "mode": self.mode}
        result = self.session.post("/api/resources", query)
        ids_names = [(resource["id"], resource["fields"]["name"]) for resource in result["resources"]]
        return ids_names

    def stream_to_server(self, id, name):
        try:
            self.content = self.session.get("/api/resources/" + id, None)["resources"]
        except requests.exceptions.HTTPError:
            self.log(f"Template with id {id} does not exist")
        else:
            for temp in self.content:
                self.log(f"moving template {name}")
                self.new_session.post("/api/resources/new", temp)


class AllUserViewTemplatesDownloader():
    def __init__(self, session, usrvw, new_session, mode, log):
        self.session = session
        self.log = log
        self.userview = usrvw
        self.content = None
        self.new_session = new_session
        self.mode = mode

    def stream_individual_templates(self):
        usrvw_dict = self.get_user_view()
        template_ids = self.userview_templates(usrvw_dict)
        for id in template_ids:
            self.stream_to_server(id)

    def userview_templates(self, usrvw_dict):
        userview_id = usrvw_dict[self.userview]
        data = {"filter": {"type": "RESOURCE_TYPES.USER_VIEW_TEMPLATES", "userViewId": f"{userview_id}"},
                "mode": self.mode, "limit": 10000, "offset": 0, "include_content": True}
        response = self.session.post('/api/resources', data)
        resources = response["resources"]
        template_ids = list()
        for temp in resources:
            for temp_id in temp.get("content", {}).get("templateIds", {}):
                template_ids.append(temp_id)
        return template_ids

    def get_user_view(self):
        usrvw_dict = dict()
        content = self.session.get("/api/user_views")
        for inf in content:
            usrvw_dict[inf["name"].lower()] = inf["id"]
        return usrvw_dict

    def stream_to_server(self, id):
        try:
            self.content = self.session.get("/api/resources/" + id, None)["resources"]
        except requests.exceptions.HTTPError:
            self.log(f"Template with id {id} does not exist")
        else:
            for temp in self.content:
                self.log(f"moving template {self.userview}")
                self.new_session.post("/api/resources/new", temp)


class AllUserViewSettingsDownloader():
    def __init__(self, session, usrvw, new_session, log):
        self.session = session
        self.log = log
        self.userview = usrvw
        self.content = None
        self.new_session = new_session

    def stream_user_view_settings(self):
        if self.userview.lower() == "default":
            usrvw_id = None
        else:
            usrvw_id = self.get_user_view()[self.userview]
            # First stream the User View if it doesn't exist yet
            self.stream_user_view(usrvw_id)
            self.log(f"User View {self.userview} streamed")
            # Second stream template availability settings for the User View
            self.stream_setting(usrvw_id, 'USER_VIEW_TEMPLATES')
            self.log(f"User View {self.userview} template availability settings streamed")

        # Third stream default template settings
        self.stream_setting(usrvw_id,'USER_VIEW_DEFAULT_TEMPLATE')
        self.log(f"User View {self.userview} default template settings streamed")

    def stream_setting(self, usrvw_id, resource_type):
        self.remove_existing_setting(usrvw_id, resource_type)
        default_ids = self.get_user_view_template_setting(usrvw_id, resource_type)
        for id in default_ids:
            content = self.session.get("/api/resources/" + id, None)["resources"]
            for temp in content:
                self.new_session.post("/api/resources/new", temp)

    def stream_user_view(self, usrvw_id):
        source_usrvw = self.session.get(
            "/api/user_views?&include_groups=true&order_by=null&direction=null&offset=0&limit=50")
        dest_usrvw = self.new_session.get(
            "/api/user_views?&include_groups=true&order_by=null&direction=null&offset=0&limit=50")
        dest_usrvw_ids = [usrvw['id'] for usrvw in dest_usrvw]
        content = [data for data in source_usrvw if data['id'] == usrvw_id and data['id'] not in dest_usrvw_ids]

        for temp in content:
            # Make sure the user groups listed exists on the destination server! If they don't exist, remove the
            # user group from the listed user view
            source_groups = temp['user_groups']
            dest_groups = self.new_session.get("/api/auth/profiles?limit=10000")
            dest_group_ids = [tag['identifier'] for tag in dest_groups['profiles']]
            temp['user_groups'] = [group for group in source_groups if group in dest_group_ids]
            self.new_session.post("/api/user_views?&include_groups=true&order_by=null&direction=null&offset=0&limit=50",
                                  temp)

    def remove_existing_setting(self, userview_id, resource_type):
        data = {"filter": {"type": "RESOURCE_TYPES." + resource_type, "userViewId": userview_id},
                "mode": "all"}
        self.new_session.post('api/resources/delete', data)

    def get_user_view(self):
        usrvw_dict = dict()
        content = self.session.get("/api/user_views")
        for inf in content:
            usrvw_dict[inf["name"].lower()] = inf["id"]
        return usrvw_dict

    def get_user_view_template_setting(self, userview_id, resource_type):
        # For default settings, userview equals "null" (not string)
        data = {"filter": {"type": "RESOURCE_TYPES." + resource_type, "userViewId": userview_id},
                "mode": "all", "limit": 10000, "offset": 0, "include_content": True}
        response = self.session.post('/api/resources', data)
        resources = response["resources"]
        setting_ids = list()
        for temp_id in resources:
            setting_ids.append(temp_id['id'])
        return setting_ids
